from django.contrib.staticfiles.management.commands.runserver import \
    Command as RunserverCommandBase
from django.core import management
from django.core.management.base import BaseCommand

from .standard import StandardCommand


class RunserverCommandMixin(object):
    """
    Runserver management command options.
    
    """
    option_list = RunserverCommandBase.option_list[len(BaseCommand.option_list):]
    option_groups = (
        ("[runserver options]",
            "These options will be passed to runserver.",
            option_list,
        ),)
    option_names = ()
    actions = ("runserver",)
    
    def handle_runserver(self):
        if self.options["verbosity"] >= 1:
            self.stdout.write(
                "\n    > runserver\n\n")
        management.call_command("runserver", *self.args, **self.options)


class RunserverCommand(RunserverCommandMixin, StandardCommand):
    """
    Runserver management command.
    
    """
    option_list = \
        StandardCommand.option_list
    option_groups = \
        RunserverCommandMixin.option_groups + \
        StandardCommand.option_groups
    option_names = \
        RunserverCommandMixin.option_names + \
        StandardCommand.option_names
    actions = \
        RunserverCommandMixin.actions + \
        StandardCommand.actions



import optparse

from django.conf import settings
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.utils.importlib import import_module

from .runserver import RunserverCommand


class GruntserverCommandMixin(object):
    """
    Gruntserver management command options.
    
    """
    option_list = (
        optparse.make_option("--no-grunt-js",
            action="store_false", dest="grunt_js", default=True,
            help="Set settings.GRUNT_JS to False",
        ),
        optparse.make_option("--no-grunt-css",
            action="store_false", dest="grunt_css", default=True,
            help="Set settings.GRUNT_CSS to False"),
        optparse.make_option("--no-gzip",
            action="store_false", dest="gzip", default=True,
            help="Do NOT add gzip middleware"),)
    option_groups = (
        ("[gruntserver options]",
            "These options will be passed to gruntserver.",
            option_list),)
    option_names = ("grunt_js", "grunt_css", "gzip",)
    actions = ("gruntserver",)
    
    def parse_option_use_build_js(self):
        return bool(self.options.get("grunt_js", True))
    
    def parse_option_use_build_css(self):
        return bool(self.options.get("grunt_css", True))
    
    def parse_option_use_gzip(self):
        return bool(self.options.get("gzip", True))
    
    def handle_gruntserver(self):
        # Set up the staticfiles app to serve files in STATIC_ROOT.
        settings.STATICFILES_FINDERS = (
            "django.contrib.staticfiles.finders.FileSystemFinder",
        )
        settings.STATICFILES_DIRS = (
            settings.STATIC_ROOT,
        )
        settings.STATIC_ROOT = ""
        
        # Add staticfiles handler to urls?
        if self.options["gzip"]:
            urlconf = import_module(settings.ROOT_URLCONF)
            urlconf.urlpatterns = staticfiles_urlpatterns() + urlconf.urlpatterns
        
        # Use gzip middleware to compress staticfiles?
        if self.options["gzip"]:
            if "django.middleware.gzip.GZipMiddleware" not in settings.MIDDLEWARE_CLASSES:
                settings.MIDDLEWARE_CLASSES = (
                    "django.middleware.gzip.GZipMiddleware",
                ) + settings.MIDDLEWARE_CLASSES
        
        # Force runserver to run with the "--nostatic" option so that the
        # middleware gets run for staticfiles requests?
        if self.options["gzip"]:
            self.options["use_static_handler"] = False
        
        # Use compiled JavaScript and CSS in the templates?
        settings.GRUNT_JS = self.options["grunt_js"]
        settings.GRUNT_CSS = self.options["grunt_css"]


class GruntserverCommand(GruntserverCommandMixin, RunserverCommand):
    """
    Command that runs the staticfiles runserver command with the
    built static files in settings.STATIC_ROOT and sets
    the values of settings.GRUNT_JS and settings.GRUNT_CSS.
    
    """
    option_list = \
        RunserverCommand.option_list
    option_groups = \
        GruntserverCommandMixin.option_groups + \
        RunserverCommand.option_groups
    option_names = \
        GruntserverCommandMixin.option_names + \
        RunserverCommand.option_names
    actions = \
        GruntserverCommandMixin.actions + \
        RunserverCommand.actions
    args = "[runserver argument ...] ([gruntserver option] | [runserver option] | [standard option])*"
    help = (
        "Starts a lightweight Web server for development and serves "
        "the Grunt-processed static files in STATIC_ROOT.")



import optparse

from .base import Command, BaseCommand
from .quiet import QuietCommand


class StandardCommand(BaseCommand, QuietCommand, Command):
    """
    Base class for enhanced, standard commands.
    
    """
    option_list = QuietCommand.option_list
    option_groups = BaseCommand.option_groups
    option_names = BaseCommand.option_names + QuietCommand.option_names
    actions = ()



import optparse
import os
import subprocess

from django.core.management.base import CommandError

from .collectstatic import CollectstaticCommand


class GruntCommandMixin(object):
    """
    Grunt management command options.
    
    """
    option_list = (
        optparse.make_option("--grunt-base",
            action="store", type="string", dest="grunt_base", default="",
            help="Specify an alternate base path for Grunt. By default, all "
                "file paths are relative to the Gruntfile."),
        optparse.make_option("--gruntfile",
            action="store", type="string", dest="gruntfile", default="",
            help="Specify an alternate Gruntfile. By default, grunt looks "
                "in the current or parent directories for the nearest "
                "Gruntfile.js or Gruntfile.coffee file."),
        optparse.make_option("--grunt-debug",
            action="store_true", dest="grunt_debug", default=False,
            help="Enable debugging mode for grunt tasks that support it."),)
    option_groups = (
        ("[grunt commands]",
            "These options will be passed on to grunt.",
            option_list),)
    option_names = ("grunt_base", "gruntfile",)
    actions = ("grunt",)
    
    def parse_option_grunt_base(self):
        return self.options.get("grunt_base", "")
    
    def parse_option_gruntfile(self):
        return self.options.get("gruntfile", "")
    
    def parse_option_grunt_debug(self):
        return bool(self.options.get("grunt_debug", False))
    
    def validate_grunt(self):
        # Is grunt available?
        with open(os.devnull, "w") as null:
            try:
                subprocess.check_call(("which", "grunt",),
                    stdout=null, stderr=null)
            except subprocess.CalledProcessError:
                raise CommandError(
                    "grunt is not available on the command line. Please ensure "
                    "that grunt is installed and reachable in your $PATH.")
    
    def handle_grunt(self):
        if self.options["verbosity"] >= 1:
            self.stdout.write(
                "\n    > grunt {args:s}\n\n".format(
                    args=" ".join(self.args)))
        
        with open(os.devnull, "w") as null:
            try:
                subprocess.check_call(
                    ("grunt",) + \
                        self.args + \
                        (("--base", self.options["grunt_base"]) \
                            if self.options["grunt_base"] \
                            else ()) + \
                        (("--gruntfile", self.options["gruntfile"]) \
                            if self.options["gruntfile"] \
                            else ()) + \
                        (("--debug",) \
                            if self.options["grunt_debug"] \
                            else ()) + \
                        (("--verbose",) \
                            if self.options["verbosity"] >= 3 \
                            else ()),
                    stdout=self.stdout if self.options["verbosity"] >= 1 else null,
                    stderr=self.stderr if self.options["verbosity"] >= 1 else null)
            except subprocess.CalledProcessError, e:
                raise CommandError(
                    "grunt failed with exit code {code:d}.".format(
                        code=e.returncode))


class GruntCommand(GruntCommandMixin, CollectstaticCommand):
    option_list = \
        CollectstaticCommand.option_list
    option_groups = \
        GruntCommandMixin.option_groups + \
        CollectstaticCommand.option_groups
    option_names = \
        GruntCommandMixin.option_names + \
        CollectstaticCommand.option_names
    actions = \
        CollectstaticCommand.actions + \
        GruntCommandMixin.actions
    args = "[grunt command ...] ([grunt option] | [collectstatic option] | [standard option])*"
    help = "Collect static files in a single location and run grunt."



import optparse

from django.contrib.staticfiles.management.commands.collectstatic import \
    Command as CollectStaticCommandBase
from django.core import management
from django.core.management.base import NoArgsCommand

from .standard import StandardCommand


class CollectstaticCommandMixin(object):
    """
    Collectstatic management command options.
    
    """
    option_list = (
        optparse.make_option("--nocollect",
            action="store_false", dest="collect", default=True,
            help="Do NOT run collectstatic"),
    ) + CollectStaticCommandBase.option_list[len(NoArgsCommand.option_list):]
    option_groups = (
        ("[collectstatic] commands",
            "These options will be passed to collectstatic.",
            option_list,
        ),)
    option_names = ("collect",)
    actions = ("collectstatic",)
    
    def parse_option_collect(self):
        return bool(self.options.get("collect", False))
    
    def handle_collectstatic(self):
        if self.options["collect"]:
            if self.options["verbosity"] >= 1:
                self.stdout.write(
                    "\n    > collectstatic\n")
            management.call_command("collectstatic", **self.options)


class CollectstaticCommand(CollectstaticCommandMixin, StandardCommand):
    """
    Collectstatic management command.
    
    """
    option_list = \
        StandardCommand.option_list
    option_groups = \
        CollectstaticCommandMixin.option_groups + \
        StandardCommand.option_groups
    option_names = \
        CollectstaticCommandMixin.option_names + \
        StandardCommand.option_names
    actions = \
        CollectstaticCommandMixin.actions + \
        StandardCommand.actions



import optparse

from django.core.management import base


class Command(base.BaseCommand):
    """
    Base class for a declarative management command API.
    
    """
    option_list = ()
    option_groups = ()
    option_names = ()
    actions = ()
    
    def usage(self, subcommand):
        """
        Override the usage display.
        
        """
        usage = "%prog {subcommand:s} {args:}".format(
            subcommand=subcommand, args=self.args)
        if self.help:
            return "{usage:s}\n\n{help:s}".format(
                usage=usage, help=self.help)
        return usage
    
    def create_parser(self, prog_name, subcommand):
        """
        Customize the OptionParser.
        
        """
        parser = super(Command, self).create_parser(prog_name, subcommand)
        for name, description, option_list in self.option_groups:
            group = optparse.OptionGroup(parser, name, description);
            map(group.add_option, option_list)
            parser.add_option_group(group)
        return parser
    
    def parse_options(self):
        for name in self.option_names:
            parse = getattr(self, "parse_option_{name:s}".format(
                name=name), None)
            if parse is not None and callable(parse):
                self.options[name] = parse()
    
    def handle(self, *args, **options):
        self.args = args
        self.options = options
        self.parse_options()
        
        for name in self.actions:
            validate = getattr(self, "validate_{name:s}".format(
                name=name), None)
            if validate is not None and callable(validate):
                validate()
        for name in self.actions:
            handle = getattr(self, "handle_{name:s}".format(
                name=name), None)
            if handle is not None and callable(handle):
                handle()


class BaseCommandMixin(object):
    """
    Base Django management command options.
    
    """
    option_list = base.BaseCommand.option_list
    option_groups = (
        ("[standard options]",
            "Standard Django management command options.",
            option_list,
        ),
    )
    option_names = ("verbosity",)
    actions = ()
    
    def parse_option_verbosity(self):
        try:
            verbosity = int(self.options.get("verbosity", 1))
        except (ValueError, TypeError):
            verbosity = 1
        return verbosity


class BaseCommand(BaseCommandMixin, Command):
    """
    Base Django management command.
    
    """
    pass



from django.conf import settings

from ..conf import grunt_conf


class GruntConfigMixin(object):
    """
    A view mixin for updating a context dictionary with grunt
    configuration data.
    
    """
    grunt_config_key = None
    
    def grunt_config(self, config=None, key=None):
        return grunt_conf(
            config={} if config is None else config,
            key=key if key is not None else self.grunt_config_key)



#!/usr/bin/env python

import os
from setuptools import setup, find_packages
import grunt


def read(filename):
    with open(os.path.join(os.path.dirname(__file__), filename)) as source:
        return source.read()


setup(
    name="django-grunt",
    version=grunt.__version__,
    description="Django tools for integrating Grunt into your workflow",
    long_description=read("README"),
    author="Mike Kibbel",
    author_email="mkibbel@gmail.com",
    url="https://github.com/skibblenybbles/django-grunt",
    license="MIT License",
    platforms=["OS Independent"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Utilities",
    ],
    install_requires=[
        "Django>=1.4",
    ],
    packages=find_packages(exclude=["example", "example.*"]),
)



import optparse

from .base import Command


class QuietCommandMixin(object):
    """
    Quiet Django management command options.
    
    """
    option_list = (
        optparse.make_option("-q", "--quiet",
            action="store_true", dest="quiet", default=False,
            help="Suppress all prompts and output."),)
    option_groups = ()
    option_names = ("quiet",)
    quiet_option_names = (
        ("interactive", False),
        ("verbosity", 0),)
    actions = ()
    
    def parse_option_quiet(self):
        quiet = bool(self.options.get("quiet", False))
        if quiet:
            for name, value in self.quiet_option_names:
                self.options[name] = value
        return quiet


class QuietCommand(QuietCommandMixin, Command):
    """
    Quiet Django management command.
    
    """
    pass



from django.conf import settings


def grunt_conf(config=None, key=None):
    """
    Returns the grunt configuration settings. Optionally updates
    the dictionary passed to config. In the output dictionary,
    optionally nests the grunt configuration settings under
    the given key.
    
    """
    if config is None:
        config = {}
    
    if key is not None:
        if key not in config:
            config[key] = {}
        config = config[key]
    
    config.update({
        "grunt_js": getattr(settings, "GRUNT_JS", False),
        "grunt_css": getattr(settings, "GRUNT_CSS", False),
    })
    
    return config
